/////////////////////////////////////////////////////////////////////////////
//
// CVideoDevice.cpp : Video Output Device 
//
// MyPhone  -  Audio/Video IP telephony client.
// Compatible with H323 protocol, based on H323plus project.
// 
// Home Page: http://www.h323plus.org/
//
// Copyright (c) 2003 Michael Gerdov aka -=MaGGuS=-
// Copyright (c) 2007 ISVO (Asia) Pte Ltd.
//
// The contents of this file are subject to the Mozilla Public License
// Version 1.0 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the License for the specific language governing rights and limitations
// under the License.
// 
// Thanks goes to All OpenH323/H323plus members (www.h323plus.org)
//				  and especially to Yuriy Gorvitovskiy (PocketBone).
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <wingdi.h>
#include "resource.h"
#include "MyPhoneDlg.h"

#include "CVideoDevice.h"

#include "Multilang.h"		// Languages


CVideoOutputDevice::CVideoOutputDevice(
	CMyPhoneDlg* wnd,
	const PString& remotePartyName,
		PBoolean fFlipVertically, 
			PBoolean fFlipHorizontally,
				PBoolean isLocal,
					PBoolean doLocalVideoPnP) :
    m_wndDlg(wnd),
	m_remotePartyName(remotePartyName),
	m_isLocal(isLocal),
	m_doLocalVideoPnP(doLocalVideoPnP),
	m_fVertFlip(fFlipVertically),
	m_fHorzFlip(fFlipHorizontally)
{

	if(m_doLocalVideoPnP || !m_isLocal)
		m_hWnd = m_wndDlg->GetDlgItem(IDC_VIDEOBITMAP);
	else if (m_isLocal)
		m_hWnd = m_wndDlg->GetDlgItem(IDC_VIDEOBITMAP1);

	m_cdc = m_hWnd->GetDC();

    m_hWnd->GetClientRect(&m_rect);
	// Set for PIP
	m_rect.right = m_doLocalVideoPnP&&m_isLocal ? m_rect.left/2 : m_rect.left;
	m_rect.bottom = m_doLocalVideoPnP&&m_isLocal ? m_rect.bottom/2 : m_rect.bottom;


	m_bmi.bmiHeader.biSize = sizeof(m_bmi.bmiHeader);
	m_bmi.bmiHeader.biWidth = frameWidth;
	m_bmi.bmiHeader.biHeight = -(int)frameHeight;
	m_bmi.bmiHeader.biPlanes = 1;
	m_bmi.bmiHeader.biBitCount = 32;
	m_bmi.bmiHeader.biCompression = BI_RGB;
	m_bmi.bmiHeader.biXPelsPerMeter = 0;
	m_bmi.bmiHeader.biYPelsPerMeter = 0;
	m_bmi.bmiHeader.biClrImportant = 0;
	m_bmi.bmiHeader.biClrUsed = 0;
	m_bmi.bmiHeader.biSizeImage = frameStore.GetSize();


	if(m_wndDlg!=NULL)
	{
		CString vStr;
		if(m_isLocal)
		{
			vStr.Format((LPCTSTR)LoadStringLang(IDS_LOCALSTR), (LPCTSTR) m_remotePartyName);
			m_wndDlg->GetDlgItem(IDC_SLVIDEO)->SetWindowText(vStr.GetBuffer(0));
		}
		else
		{
			vStr.Format((LPCTSTR)LoadStringLang(IDS_REMOTESTR), (LPCTSTR) m_remotePartyName);
			m_wndDlg->GetDlgItem(IDC_SRVIDEO)->SetWindowText(vStr.GetBuffer(0));
		}
	}

}

CVideoOutputDevice::~CVideoOutputDevice()
{
  PWaitAndSignal m(mutex);
  Close();
}

PBoolean CVideoOutputDevice::Open(const PString & /*deviceName*/,
                              PBoolean /*startImmediate*/)
{
  return TRUE;
}

PBoolean CVideoOutputDevice::IsOpen()
{
  return m_hWnd != NULL;
}

PBoolean CVideoOutputDevice::Close()
{
  return TRUE;
}

PStringArray CVideoOutputDevice::GetDeviceNames() const
{
  PStringList  devlist;
  devlist.AppendString(GetDeviceName());
  return devlist;
}

PBoolean CVideoOutputDevice::FrameComplete()
{
  PWaitAndSignal m(mutex);

  if (m_hWnd == NULL)
    return FALSE;

  Redraw(m_cdc->GetSafeHdc());

  return TRUE;
}


void CVideoOutputDevice::Redraw(HDC curDC)
{
	if(curDC)
	{
		::SetStretchBltMode(curDC, HALFTONE);

		// Important note: 
		// Images height is measured from the bottom in the framestore

		int result=0;
		if (m_doLocalVideoPnP && !m_isLocal && !m_wndDlg->hidePnP)
		{
			// Upper right part of image
			result = StretchDIBits(curDC, 
				frameWidth/2, 0, frameWidth/2, frameHeight/2 , 
				frameWidth/2, frameHeight/2, frameWidth/2, frameHeight/2, 
				frameStore.GetPointer(), &m_bmi, DIB_RGB_COLORS, SRCCOPY);
			// bottom full width part of image
			result = StretchDIBits(curDC, 
				0, frameHeight/2, frameWidth, frameHeight/2 , 
				0, 0, frameWidth, frameHeight/2, 
				frameStore.GetPointer(), &m_bmi, DIB_RGB_COLORS, SRCCOPY);
		}
		else if (m_doLocalVideoPnP && m_isLocal && !m_wndDlg->hidePnP) 
		{
			result = StretchDIBits(curDC, 
				0, 0, frameWidth/2, frameHeight/2, 
				0, 0, frameWidth, frameHeight, 
				frameStore.GetPointer(), &m_bmi, DIB_RGB_COLORS, SRCCOPY);

		} else if (!m_isLocal || (m_isLocal && !m_doLocalVideoPnP)) 
		{
			result = SetDIBitsToDevice(curDC,
                               0, 0, frameWidth, frameHeight,
                               0, 0, 0, frameHeight,
                               frameStore.GetPointer(),  &m_bmi, DIB_RGB_COLORS);
		}

        if (result == GDI_ERROR) {
             lastError = ::GetLastError();
             PTRACE(2, "VidOut\tDrawing image failed, error=" << lastError);
		} 
	}

}

PBoolean CVideoOutputDevice::SetFrameSize( unsigned width, unsigned height )
{
	if (width == 0 || height == 0)
		return TRUE;

    if (m_hWnd != NULL && width == frameWidth && height == frameHeight)
      return TRUE;

	PTRACE(4,"VID\tSetting Frame Size w " << width << " h " << height);

    if (!PVideoOutputDeviceRGB::SetFrameSize(width, height))
      return FALSE;

    m_bmi.bmiHeader.biWidth = frameWidth;
    m_bmi.bmiHeader.biHeight = m_fVertFlip ? frameHeight : -(int)frameHeight;
    m_bmi.bmiHeader.biSizeImage = frameStore.GetSize();

	return TRUE;
}

PBoolean CVideoOutputDevice::SetColourFormat(const PString & colourFormat)
{
  PWaitAndSignal m(mutex);

  if (((colourFormat *= "BGR24") || (colourFormat *= "BGR32")) &&
                PVideoOutputDeviceRGB::SetColourFormat(colourFormat)) {
    m_bmi.bmiHeader.biBitCount = (WORD)(bytesPerPixel*8);
    m_bmi.bmiHeader.biSizeImage = frameStore.GetSize();
    return TRUE;
  }

  return FALSE;
}
