/////////////////////////////////////////////////////////////////////////////
//
// CVideoDevice.h : interface for Video Output Device
//
// MyPhone  -  Audio/Video IP telephony client.
// Compatible with H323 protocol, based on H323plus project.
// 
// Home Page: http://www.h323plus.org/
//
// Copyright (c) 2003 Michael Gerdov aka -=MaGGuS=-
// Copyright (c) 2007 ISVO (Asia) Pte Ltd.
//
// The contents of this file are subject to the Mozilla Public License
// Version 1.0 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the License for the specific language governing rights and limitations
// under the License.
// 
// Thanks goes to All OpenH323/H323plus members (www.h323plus.org)
//				  and especially to Yuriy Gorvitovskiy (PocketBone).
//
///////////////////////////////////////////////////////////////////////////////

#ifndef CVIDEODEVICE_H
#define CVIDEODEVICE_H

#include <ptlib.h>
#include <ptlib/video.h>

//#include "MyPhoneDlg.h"
//#include <codecs.h>

class CVideoOutputDevice : public PVideoOutputDeviceRGB
{
public:

	CVideoOutputDevice(CMyPhoneDlg* wnd,
		const PString& remotePartyName, 
		PBoolean fFlipVertically = FALSE, 
		PBoolean fFlipHorizontally = FALSE,
		PBoolean isLocal = FALSE,
		PBoolean doLocalVideoPnP = FALSE);

	~CVideoOutputDevice();

    virtual PBoolean Open(
      const PString & deviceName,   /// Device name to open
      PBoolean startImmediate = TRUE    /// Immediately start device
	  );

    virtual PBoolean IsOpen();

	virtual PBoolean Close();

    virtual PStringArray GetDeviceNames() const;

	// Resizing frames
	PBoolean SetFrameSize(
      unsigned width,   /// New width of frame
      unsigned height   /// New height of frame
    );

    PBoolean SetColourFormat(
      const PString & colourFormat // New colour format for device.
    );

    PBoolean FrameComplete();

	void Redraw(HDC curDC);

protected:
	CMyPhoneDlg*	m_wndDlg;
	CWnd*			m_hWnd;
    CDC*            m_cdc;
	PString			m_remotePartyName;
	RECT            m_rect;
	BITMAPINFO		m_bmi;
    PMutex			mutex;

	PBoolean			m_fVertFlip;
	PBoolean			m_fHorzFlip;
	PBoolean			m_isLocal;
	PBoolean			m_doLocalVideoPnP;

};

#endif
