///////////////////////////////////////////////////////////////////////////////
// MyPhoneEndPoint.cpp: implementation for the CMyPhoneEndPoint class.
//
// MyPhone  -  Audio/Video IP telephony client.
// Compatible with H323 protocol, based on H323plus project.
// 
// Home Page: http://www.h323plus.org/
//
// Copyright (c) 2003 Michael Gerdov aka -=MaGGuS=-
// Copyright (c) 2007 ISVO (Asia) Pte Ltd.
//
// The contents of this file are subject to the Mozilla Public License
// Version 1.0 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the License for the specific language governing rights and limitations
// under the License.
// 
// Thanks goes to All OpenH323/H323plus members (www.h323plus.org)
//				  and especially to Yuriy Gorvitovskiy (PocketBone).
//
///////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"

#include "MyPhone.h"
#include "MyPhoneEndPoint.h"
#include "MyPhoneDlg.h"
#include "CVideoDevice.h"

#include "Multilang.h"		// Languages


#if defined(P_DIRECTSHOW2)
  PString CMyPhoneEndPoint::VideoInputDriver = "DirectShow";
#else
  PString CMyPhoneEndPoint::VideoInputDriver = "VideoForWindows";
#endif

/////////////////////////// Config Keys
const char Ready[] = "Ready. IP address is ";

const char DtmfAsStringConfigKey[] = "DtmfAsString";
const char BandwidthConfigKey[] = "Bandwidth";
const char ListenerInterfaceConfigKey[] = "ListenerInterface";
const char RouterConfigKey[] = "NATRouterAddress";

const char RTPPortBaseConfigKey[] = "RTPPortBase";
const char RTPPortMaxConfigKey[] = "RTPPortMax";

const char NoTunnelingConfigKey[] = "NoTunneling";
const char NoFastStartConfigKey[] = "FastStart";
const char IpTosConfigKey[] = "IpTOS";
const char UserInputModeConfigKey[] = "UserInputMode";
const char UsernameConfigKey[] = "Username";

const char AliasConfigKey[] = "UserAliases";

const char AutoAnswerConfigKey[] = "AutoAnswer";

const char CodecsConfigSection[] = "Codecs";
const char OnCodecSuffix[] = " (On)";
const char OffCodecSuffix[] = " (Off)";

const char DiscoverGatekeeperConfigKey[] = "DiscoverGatekeeper";
const char UseGatekeeperConfigKey[] = "UseGatekeeper";
const char RequireGatekeeperConfigKey[] = "RequireGatekeeper";
const char TimeToLiveConfigKey[] = "TimeToLive";
const char GatekeeperHostConfigKey[] = "GatekeeperHost";
const char GatekeeperIdConfigKey[] = "GatekeeperId";
const char GatekeeperPassConfigKey[] = "GatekeeperPassword";

const char SoundPlayConfigKey[] = "SoundPlayDevice";
const char SoundRecordConfigKey[] = "SoundRecordDevice";
const char SilenceDetectConfigKey[] = "SilenceDetect";
const char JitterConfigKey[] = "Jitter";
const char BufferCountConfigKey[] = "BufferCount";

const char H261QCIFConfigKey[] = "H261_QCIF";
const char H261CIFConfigKey[] = "H261_CIF";

const char AutoTransmitVideoConfigKey[] = "AutoTransmitVideo";
const char AutoReceiveVideoConfigKey[] = "AutoReceiveVideo";
const char VideoCodecsConfigSection[] = "VideoCodecs";
const char VideoDeviceConfigKey[] = "VideoDevice";
const char VideoSizeConfigKey[] = "VideoSize";
const char VideoSourceConfigKey[] = "VideoSource";
const char VideoFormatConfigKey[] = "VideoFormat";
const char VideoLocalConfigKey[] = "VideoLocal";
const char VideoFlipLocalConfigKey[] = "VideoFlipLocal";
const char VideoQualityConfigKey[] = "VideoQuality";
const char VideoMaxbandWidthKey[] = "VideoMaxbandwidth";
const char VideoFPSKey[] = "VideoFramesPerSecond";
const char VideoPacketSizeConfigKey[] = "VideoPacketSize";
const char VideoInVFlipConfigKey[] = "VideoInVFlip";
const char VideoInHFlipConfigKey[] = "VideoInHFlip";
const char VideoOutVFlipConfigKey[] = "VideoOutVFlip";
const char VideoOutHFlipConfigKey[] = "VideoOutHFlip";
const char VideoSmartCodecConfigKey[] = "VideoSmartCodecMode";

//////////////////////////////////////////////////////////////////////
// Derived Connection
CMyPhoneConnection::CMyPhoneConnection(
									   CMyPhoneDlg* wnd, CMyPhoneEndPoint & ep, unsigned callReference, unsigned options)
									   : H323Connection(ep, callReference, options), endpoint(ep), m_dialog(wnd)
{
}

PBoolean CMyPhoneConnection::OnAlerting(const H323SignalPDU &, const PString & user)
{
	CString fixuser = m_dialog->FindContactName(*this);
	m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_RINGSTR), S_SYSTEM, (const char *)fixuser);
	CString allStr;
	allStr.Format((LPCTSTR)LoadStringLang(IDS_RING1STR), (const char*) fixuser);
	m_dialog->m_caller.SetWindowText((LPCTSTR)allStr);
	return TRUE;
}


void CMyPhoneConnection::OnUserInputString(const PString & value)
{
	m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_REPLMSGSTR), S_INMSG, 
		m_dialog->FindContactName(*this)/*(const char *) this->GetRemotePartyName()*/, 
		(const char *) value);
} 

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PluginLoaderStartup2  CMyPhoneApp::PWLibProcess::pluginLoader;
H323PluginCodecManager * CMyPhoneApp::PWLibProcess::plugmgr=NULL;


CMyPhoneEndPoint::CMyPhoneEndPoint()
{
	m_dialog = NULL;
	m_stat.iSecs=0;  m_stat.ibSent=0;  m_stat.ibRcvd=0;  m_stat.iDelay=0;
	videoSize = 0;
	m_router = NULL;
	localVideo = FALSE;
	videoFramesPS = 0;
	videoBitRate = 0;

	// MFC applications are not at all plugin friendly
	// You need to manually add the plugins
	CMyPhoneApp::PWLibProcess::LoadPluginMgr();
	CMyPhoneApp::PWLibProcess::pluginLoader.OnStartup();
}

CMyPhoneEndPoint::~CMyPhoneEndPoint()
{
	// You need to manually remove the plugins
	CMyPhoneApp::PWLibProcess::RemovePluginMgr();
	CMyPhoneApp::PWLibProcess::pluginLoader.OnShutdown();
}

PBoolean CMyPhoneEndPoint::Initialise(CMyPhoneDlg *dlg)
{
	// Load the base featureSet
    LoadBaseFeatureSet();

	m_dialog = dlg;
	
	SetAudioJitterDelay(50, config.GetInteger(JitterConfigKey, GetMaxAudioJitterDelay()));
	SetSoundChannelBufferDepth(config.GetInteger(BufferCountConfigKey, GetSoundChannelBufferDepth()));
	
	// UserInput mode
    // Backward compatibility configuration entry
    unsigned mode = H323Connection::SendUserInputAsString;
	m_fDtmfAsString=true;
    if (config.HasKey(DtmfAsStringConfigKey)) {
		if (!config.GetBoolean(DtmfAsStringConfigKey))
		{
			mode = H323Connection::SendUserInputAsTone;
			m_fDtmfAsString=false;
		}
		config.DeleteKey(DtmfAsStringConfigKey);
	}
	mode = config.GetInteger(UserInputModeConfigKey, mode);
	SetSendUserInputMode((H323Connection::SendUserInputModes)mode);
	
	// set record and play devices
	PBoolean fPlayerAvailable = SetSoundChannelPlayDevice(
		config.GetString(SoundPlayConfigKey,   GetSoundChannelPlayDevice()));
	PBoolean fRecorderAvailable = SetSoundChannelRecordDevice(
		config.GetString(SoundRecordConfigKey, GetSoundChannelRecordDevice()));
	
	// set some oter settings from Config
	m_fNoFastStart =  config.GetBoolean(NoFastStartConfigKey, FALSE);
    DisableFastStart(m_fNoFastStart);
	m_fDoH245Tunnelling = !(config.GetBoolean(NoTunnelingConfigKey, FALSE));
    DisableH245Tunneling(!m_fDoH245Tunnelling);
	m_fSilenceOn = config.GetBoolean(SilenceDetectConfigKey, TRUE);
    SetSilenceDetectionMode(m_fSilenceOn
		? H323AudioCodec::AdaptiveSilenceDetection
		: H323AudioCodec::NoSilenceDetection);
    SetLocalUserName(config.GetString(UsernameConfigKey, GetLocalUserName()));
    SetInitialBandwidth((unsigned)(config.GetReal(BandwidthConfigKey, 10000)*20));
    SetRtpIpTypeofService(config.GetInteger(IpTosConfigKey, GetRtpIpTypeofService()));
	
	SetRtpIpPorts(config.GetInteger(RTPPortBaseConfigKey, GetRtpIpPortBase()),
		config.GetInteger(RTPPortMaxConfigKey,  GetRtpIpPortBase()));
	if(config.HasKey(RouterConfigKey))
		m_router = config.GetString(RouterConfigKey, m_router.AsString());
	
	m_fAutoAnswer = config.GetBoolean(AutoAnswerConfigKey, false);
	
    CString alias, aliases;
	aliases = CString((const char *)config.GetString(AliasConfigKey, _T("")));
	aliases.TrimLeft();
	int iPos=0;
    while ((iPos = aliases.Find(_T("|")))>0)  // loading user aliases
	{
		alias = aliases.Left(iPos);
		aliases.Delete(0,iPos+1);
		AddAliasName((LPCTSTR)alias);
	}
	
	// The order in which capabilities are added to the capability table
	// determines which one is selected by default.
	LoadCapabilities();
	
	PString interfaces = config.GetString(ListenerInterfaceConfigKey, "*");
	if(StartListeners(interfaces.Tokenise(',')))
	{
		m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_MPREADY1STR), S_SYSTEM, (const char *)strProgName);
		return true;
	}
	else
	{
		m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRLSNSTR), S_SYSTEM, (const char *)strProgName, (const char *)interfaces);
		return false;
	}
}

H323Connection * CMyPhoneEndPoint::CreateConnection(unsigned int refID)
{
	// close video panel before everything else
	if(m_dialog->autohideVideoPan && m_dialog->showVideoPan )
		m_dialog->ShowVideoPanels(FALSE);  // hide Video panel if it's visible
	
	return new CMyPhoneConnection(m_dialog, *this, refID, 0);
}

PBoolean CMyPhoneEndPoint::OnIncomingCall(H323Connection & connection,
									  const H323SignalPDU &,
									  H323SignalPDU &)
{
	CTime tNow;
	tNow=CTime::GetCurrentTime();
	return TRUE;
}

void CMyPhoneEndPoint::OnConnectionEstablished(H323Connection & connection, const PString & token)
{
	m_dialog->m_token = token;
	CString goodName = (LPCTSTR)connection.GetRemotePartyName();
	// store last number in combobox
	if(!connection.HadAnsweredCall())
	{
		m_dialog->PhoneBookAddCall(1, CString((LPCTSTR)connection.GetRemotePartyName()), m_dialog->GetSimpleAdr(CString((LPCTSTR)connection.GetRemotePartyAddress())));
		m_dialog->StoreDestonation(goodName /*(const char *)connection.GetRemotePartyName()*/);
	}
	else
	{
		// store in Address Book
		m_dialog->PhoneBookAddCall(2, CString((LPCTSTR)connection.GetRemotePartyName()), m_dialog->GetSimpleAdr(CString((LPCTSTR)connection.GetRemotePartyAddress())));
		if(m_dialog->autoAddInAddr)
			m_dialog->StoreDestonation(goodName /*(const char *)connection.GetRemotePartyName()*/, TRUE);
	}
	m_dialog->OnConnectionEstablished(m_dialog->FindContactName(connection) /*(const char *)connection.GetRemotePartyName()*/);
}

void CMyPhoneEndPoint::OnConnectionCleared(H323Connection & connection, const PString & clearedCallToken)
{
	if(connection.GetCallEndReason()==H323Connection::EndedByRemoteBusy)
		m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_CALLBUSYSTR), S_SYSTEM, m_dialog->FindContactName(connection));

	// clearing statistic
	m_stat.iSecs=0;
	m_stat.ibSent=0;
	m_stat.ibRcvd=0;
	m_stat.iDelay=0;
	// UI change
	m_dialog->OnConnectionCleared(m_dialog->FindContactName(connection));
}

H323Connection::AnswerCallResponse CMyPhoneEndPoint::OnAnswerCall(H323Connection & connection, const PString &caller, const H323SignalPDU &, H323SignalPDU &)
{
	return m_dialog->OnAnswerCall(connection) ? H323Connection::AnswerCallNow : H323Connection::AnswerCallPending;
}

PBoolean CMyPhoneEndPoint::OpenAudioChannel(H323Connection & connection,
										PBoolean isEncoding,
										unsigned bufferSize,
										H323AudioCodec & codec)
{
	return H323EndPoint::OpenAudioChannel(connection, isEncoding, bufferSize, codec);
}

PBoolean CMyPhoneEndPoint::OnStartLogicalChannel(H323Connection &, H323Channel & channel)
{
	OnLogicalChannel(channel, IDS_STARTCHANNELTX, IDS_STARTCHANNELRX);
	return TRUE;
}

void CMyPhoneEndPoint::OnLogicalChannel(const H323Channel & channel, unsigned txStrID, unsigned rxStrID)
{
	const H323Capability & capability = channel.GetCapability();
	PString name = capability.GetFormatName();
	PString frames;

	if (capability.GetMainType() == H323Capability::e_Video)
	{
		unsigned numFrames = channel.GetDirection() == H323Channel::IsTransmitter
			? capability.GetTxFramesInPacket()
			: capability.GetRxFramesInPacket();
		frames.sprintf(" (%u frames)", numFrames);
	}
	else if (capability.GetMainType() == H323Capability::e_Video)
	{
		frames.sprintf(" (%ux%u pixels)", videoWidth, videoHeight);
	}
	
	switch (channel.GetDirection())
	{
    case H323Channel::IsTransmitter :
		m_dialog->OutputStatus(txStrID, (const char *)name, (const char *)frames);
		break;
		
    case H323Channel::IsReceiver :
		m_dialog->OutputStatus(rxStrID, (const char *)name, (const char *)frames);
		break;
		
    default :
		break;
	}
}

void CMyPhoneEndPoint::OnClosedLogicalChannel(H323Connection &, const H323Channel & channel)
{
	OnLogicalChannel(channel, IDS_STOPCHANNELTX, IDS_STOPCHANNELRX);
}

void CMyPhoneEndPoint::LoadCapabilities()
{

PBoolean sizeChange = FALSE;

// Only load the codecs at startup!
// This speeds up the reload after settings change
 if (videoSize ==0 /* != config.GetInteger(VideoSizeConfigKey, 2)*/) {
  
	capabilities.RemoveAll();
	
// Add the codecs we know about
    AddAllCapabilities(0, 0, "*"); 
// Add all the UserInput capabilities
    AddAllUserInputCapabilities(0, 1);

 } 
 else if (videoSize != config.GetInteger(VideoSizeConfigKey, 2)) {
	 sizeChange = TRUE;
 }

 videoSize = config.GetInteger(VideoSizeConfigKey, 2);

#ifndef NO_H323_VIDEO
	RemoveCapability(H323Capability::e_ExtendVideo);
	switch (videoSize) 
	{
	
    case 2 : //QCIF
	//	SetVideoFrameSize(H323Capability::qcifMPI);
		videoWidth = 176;
		videoHeight = 144;
		break;

    case 3 : //CIF
	//	SetVideoFrameSize(H323Capability::cifMPI);
		videoWidth = 352;
		videoHeight = 288;
		break;
		
    case 1 :
		videoWidth = 50;  //autosize
		videoHeight = 50;  //autosize
		
	} 
#endif

	autoStartTransmitVideo = config.GetBoolean(AutoTransmitVideoConfigKey, TRUE);
	autoStartReceiveVideo = config.GetBoolean(AutoReceiveVideoConfigKey, TRUE);
	
	localVideo = config.GetBoolean(VideoLocalConfigKey, FALSE);
	localFlip = config.GetBoolean(VideoFlipLocalConfigKey, FALSE);
	
	int maxbw = config.GetInteger(VideoMaxbandWidthKey, 320);
	
	videoBitRate = config.GetInteger(VideoMaxbandWidthKey, 320);
	videoBitRate = 1024 * PMAX(16, PMIN(2048, videoBitRate));

	videoFramesPS = config.GetInteger(VideoFPSKey, 10);
//	if (videoFramesPS >0 && videoFramesPS<30)
//		videoFramesPS = 1000/videoFramesPS;
//	else
//		videoFramesPS = 0;

    // changing audio codecs
    PStringArray enabledCodecs;
	PINDEX codecNum = 0;
	for (;;) 
	{
		PString key = psprintf("%u", ++codecNum);
		PString name = config.GetString(CodecsConfigSection, key, "");
		if (name.IsEmpty())
			break;
		
		PINDEX suffixPos = name.Find(OffCodecSuffix);
		if (suffixPos != P_MAX_INDEX) 
		{
			capabilities.Remove(name(0, suffixPos-1));
			continue;
		}
		
		suffixPos = name.Find(OnCodecSuffix);
		if (suffixPos != P_MAX_INDEX)
			name.Delete(suffixPos, P_MAX_INDEX);
		
		enabledCodecs.AppendString(name);
	}

	// changing video codecs
	codecNum = 0;
	if (sizeChange) {
		switch (videoSize) {
		case 2:
			enabledCodecs.AppendString("-QCIF");
			break;
		case 3:
  			enabledCodecs.AppendString("-CIF");
			break;
		default:
			break;
		}
	}
	for (;;) 
	{
		PString key = psprintf("%u", ++codecNum);
		PString name = config.GetString(VideoCodecsConfigSection, key, "");
		if (name.IsEmpty())
			break;
		
		PINDEX suffixPos = name.Find(OffCodecSuffix);
		if (suffixPos != P_MAX_INDEX) 
		{
			capabilities.Remove(name(0, suffixPos-1));
			continue;
		}
		
		suffixPos = name.Find(OnCodecSuffix);
		if (suffixPos != P_MAX_INDEX)
			name.Delete(suffixPos, P_MAX_INDEX);
		
		enabledCodecs.AppendString(name);
	}

	// Reorder the codecs we have
	capabilities.Reorder(enabledCodecs);
	
	PINDEX audioNum = 1;
	PINDEX videoNum = 1;
	for (PINDEX i = 0; i < capabilities.GetSize(); i++) 
	{
		if (capabilities[i].GetMainType() == H323Capability::e_Audio)
		{
			config.SetString(CodecsConfigSection,
				psprintf("%u", audioNum++),
				capabilities[i].GetFormatName() + OnCodecSuffix);
		} 
		else if (capabilities[i].GetMainType() == H323Capability::e_Video)
		{
			config.SetString(VideoCodecsConfigSection,
				psprintf("%u", videoNum++),
				capabilities[i].GetFormatName() + OnCodecSuffix);
		}
	}


	PTRACE(1, "MyPhone\tCapability Table:\n" << setprecision(4) << capabilities);
}

PBoolean CMyPhoneEndPoint::OpenVideoChannel(H323Connection & connection,
										PBoolean isEncoding,
										H323VideoCodec & codec)
{

	PVideoChannel   * channel = new PVideoChannel;
	PVideoDevice * displayDevice = NULL;
	
	if (isEncoding) 
	{
		// Transmitter part
		if(!autoStartTransmitVideo)
			return FALSE;
		
		codec.SetTxQualityLevel(config.GetInteger(VideoQualityConfigKey,15));
		codec.SetBackgroundFill(2);
	    codec.SetVideoMode(H323VideoCodec::AdaptivePacketDelay);
	    codec.SetTargetFrameTimeMs(videoFramesPS ? 1000/videoFramesPS : 0);

	    if (videoBitRate != 0)
		  codec.SetMaxBitRate(videoBitRate * 1024);  

		//Create grabber.
		bool NoDevice = false;
		PString deviceName = config.GetString(VideoDeviceConfigKey, deviceName);
		if (deviceName.IsEmpty()) {
            PStringArray devices = PVideoInputDevice::GetDriversDeviceNames(VideoInputDriver);
			if (!devices.IsEmpty())
				deviceName = devices[0];
			else 
				NoDevice = true;
		}
		
		PVideoInputDevice * grabber = NULL;
		if (deviceName.Find("fake") == 0)
			NoDevice = true;
		else 
			grabber = PVideoInputDevice::CreateDeviceByName(deviceName,VideoInputDriver);

		
		if (NoDevice || 
			!grabber->SetColourFormatConverter("YUV420P") ||
			!grabber->SetFrameSize(videoWidth, videoHeight) ||
			!grabber->SetFrameRate(videoFramesPS) ||
			!grabber->Open(deviceName, FALSE)) 
		{
			if(!NoDevice)
			{
				char sSrc[64];
				m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRVDEVSTR), S_SYSTEM, 
					(const char *) deviceName, 
					_itoa(config.GetInteger(VideoSourceConfigKey,0), sSrc, 10));
				PTRACE(1, "Failed to open or configure the video device \"" << deviceName << '"');
			}

			if(grabber) delete grabber;
			grabber = PVideoInputDevice::CreateDevice("FakeVideo");
			grabber->SetColourFormat("YUV420P");
			grabber->SetVideoFormat(PVideoDevice::PAL);
			grabber->SetFrameSize(videoWidth, videoHeight);
			grabber->SetVFlipState(localFlip);
			grabber->SetChannel(4);
		}
		
		grabber->Start();
		
		channel->AttachVideoReader(grabber);
		
		if (localVideo)
		{
			PBoolean curVFlip = config.GetBoolean(VideoOutVFlipConfigKey, FALSE);
			PBoolean curHFlip = config.GetBoolean(VideoOutHFlipConfigKey, FALSE);
			displayDevice = new CVideoOutputDevice(m_dialog, 
				connection.GetLocalPartyName(), curVFlip, curHFlip, TRUE, m_dialog->curVideoSize==CIF);
		}
		else 
			displayDevice = PVideoOutputDevice::CreateDevice("NULLOutput");
	}
	else
	{
		// Receiver part
		if(!autoStartReceiveVideo)
			return FALSE;
		PBoolean curVFlip = config.GetBoolean(VideoInVFlipConfigKey, FALSE);
		PBoolean curHFlip = config.GetBoolean(VideoInHFlipConfigKey, FALSE);

		displayDevice = new CVideoOutputDevice( 
			m_dialog, (LPCTSTR)m_dialog->FindContactName(connection), curVFlip,
			curHFlip, FALSE, m_dialog->curVideoSize==CIF && localVideo);
	}
	
	if(displayDevice)
	{
		if( m_dialog->autohideVideoPan && !m_dialog->showVideoPan)
			m_dialog->ShowVideoPanels(TRUE);  // show Video panel if it's hiden
		displayDevice->SetColourFormat("RGB32");
		displayDevice->SetColourFormatConverter("YUV420P");
	    displayDevice->SetFrameSize(videoWidth, videoHeight);
		//Give the video window refreshing class to the channel.
		channel->AttachVideoPlayer((PVideoOutputDevice *)displayDevice);
	}
	
	return codec.AttachChannel(channel,TRUE);
}

void CMyPhoneEndPoint::OnRTPStatistics(const H323Connection & connection,
									   const RTP_Session & session) const
{
	// Lock mutex
	if(!(((CMyPhoneEndPoint *)this)->m_StatLock.Lock(10))) return;
	// Reset statistic 
	m_stat.ibSent = m_stat.ibRcvd =0;
    // Getting Audio statistic part
	RTP_Session * asession = connection.GetSession(RTP_Session::DefaultAudioSessionID);
	if (asession != NULL)
	{
		int abSent=0, abRcvd=0;
		// If not sending or receiving Audio, memory read error happens.
		try{	abSent = asession->GetOctetsSent();	} catch(...) {abSent = 0;}
		try{	abRcvd = asession->GetOctetsReceived(); } catch(...) {abRcvd = 0;}
		m_stat.ibSent = abSent;
		m_stat.ibRcvd = abRcvd;
	}
	
	// Getting Video statistic part
	RTP_Session * vsession = connection.GetSession(RTP_Session::DefaultVideoSessionID);
	if (vsession != NULL)
	{
		int vbSent=0, vbRcvd=0;
		// If not sending or receiving Vidio 
		// mamory read error happends.
		try{	vbSent = vsession->GetOctetsSent();	} catch(...) {vbSent = 0;}
		try{	vbRcvd = vsession->GetOctetsReceived(); } catch(...) {vbRcvd = 0;}
		m_stat.ibSent += vbSent;
		m_stat.ibRcvd += vbRcvd;
	}
	
	m_stat.iDelay = connection.GetRoundTripDelay().GetInterval();
	
	m_stat.iSecs = (PTime() - connection.GetConnectionStartTime()).GetSeconds();
	//
	// Release mutex
	((CMyPhoneEndPoint *)this)->m_StatLock.Unlock();
}

PBoolean CMyPhoneEndPoint::FindGatekeeper()
{
	if (GetGatekeeper() != NULL) {
		if (gatekeeper->IsRegistered()) // If registered, then unregister
			m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKUNREGSTR), S_SYSTEM, (const char *)gatekeeper->GetName());	
		RemoveGatekeeper();
	}
	
	if (!config.GetBoolean(UseGatekeeperConfigKey, FALSE))
		return TRUE;
	
	SetGatekeeperPassword(config.GetString(GatekeeperPassConfigKey));
	
	PString gkHost = config.GetString(GatekeeperHostConfigKey, "");
	PString gkid = config.GetString(GatekeeperIdConfigKey, "");
	PString iface = "";
	int iMode = config.GetInteger(DiscoverGatekeeperConfigKey, -1);
	switch(iMode)
	{
	case -1:
		return TRUE;
		break;
	case 0:
		gkHost = "";
		gkid = "";
		iface = "";
		break;
	case 1:
		gkid = "";
		iface = "";
		break;
	case 2:
		gkHost = "";
		iface = "";
		break;
	}

	m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKSRCHSTR), S_SYSTEM);	
	m_dialog->m_cstatus.UpdateWindow();
	AfxGetApp()->DoWaitCursor(1);
	if (UseGatekeeper(gkHost, gkid, iface)) {
		m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKREGSTR), S_SYSTEM, (const char *)gatekeeper->GetName());	
	}
	AfxGetApp()->DoWaitCursor(0);
	
	PBoolean gkRequired = config.GetBoolean(RequireGatekeeperConfigKey, FALSE);
	
	if (GetGatekeeper() != NULL)
	{
		unsigned reason = gatekeeper->GetRegistrationFailReason();
		switch (reason) {
		case H323Gatekeeper::InvalidListener :
			m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRSTR),S_SYSTEM, (LPCTSTR)LoadStringLang(IDS_GKERRPORTSTR));
			break;
		case H323Gatekeeper::DuplicateAlias :
			m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRSTR),S_SYSTEM, (LPCTSTR)LoadStringLang(IDS_GKERRALIASSTR));
			break;
		case H323Gatekeeper::SecurityDenied :
			m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRSTR),S_SYSTEM, (LPCTSTR)LoadStringLang(IDS_GKERRSECSTR));
			break;
		case H323Gatekeeper::TransportError :
			m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRSTR),S_SYSTEM, (LPCTSTR)LoadStringLang(IDS_GKERRTRNSTR));
			break;
		default :
			if ((reason&H323Gatekeeper::RegistrationRejectReasonMask) != 0)
			{
				CString strRes;
				strRes.Format((LPCTSTR)LoadStringLang(IDS_GKERRUNSTR),(int) reason&(H323Gatekeeper::RegistrationRejectReasonMask-1));
				m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_ERRSTR), S_SYSTEM, (LPCTSTR) strRes);
			}
			break;
		}
	}
	else
	{
		if (!gkHost.IsEmpty())
		{
			if (!gkid.IsEmpty())
				m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKERRFINDSTR),
				S_SYSTEM, (const char *)gkid, (const char *)gkHost);
			else
				m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKERRFIND1STR),
				S_SYSTEM, (const char *)gkHost);
		}
		else {
			if (!gkid.IsEmpty())
				m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKERRFIND2STR),
				S_SYSTEM, (const char *)gkid);
			else
			{
				if (gkRequired)
					m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKERRFIND3STR),
					S_SYSTEM);
				else
					m_dialog->OutputStatusStr((LPCTSTR)LoadStringLang(IDS_GKERRFIND4STR),
					S_SYSTEM);
			}
		}
	}
	
	return !gkRequired; 
}

void CMyPhoneEndPoint::TranslateTCPAddress(PIPSocket::Address & localAddr,
										   const PIPSocket::Address & remoteAddr)
{
	if ((m_router!=NULL) && m_router.IsValid() && IsLocalAddress(localAddr) && !IsLocalAddress(remoteAddr))
		localAddr = m_router;
}
